%-----------------------------------------------------------------------
% 1D FEM WITH ADAPTIVE RE-MESHING
% Explicit and Averaging type error estimates implemented
% Coded by Gautam Ethiraj in Dec 2007, Swansea, Wales, UK
%-----------------------------------------------------------------------

clc
clf

%User input 
nelem = input('number of elements= ');      %No of elements
N = nelem;
h = 1/nelem;                                %mesh size    
ch = input('type of integration? ( 0- Matlab routine, 1- 1 Point Gaussian Quad, 2- 2 Point Gaussian Quad.): ');
err = input('What type of error estimate? (1- Explicit residual, 2- averaging type): ');

%nodal coordinates in xnodes
xnodes = zeros(1,nelem+1);  
for i=0:1:nelem,
    xnodes(i+1) = i*h;
end

TOL = input('maximum |||e||| allowed = ');
%setting tolerance at a value above EN for first step to enter loop
EN = (TOL +1);      

while EN > TOL
%Location Matrix for assembly information
    LM = zeros(2,nelem);   
for e=1:1:nelem,
    LM(1,e) = e;
    LM(2,e) = e+1;
end
%---------------------------STIFFNESS MATRIX------------------------------%
K = zeros(nelem+1,nelem+1);
for e=1:1:nelem,
    %element stiffness matrix
    ke = elemstiff1D(ch,xnodes(e),xnodes(e+1));
    %add element contributions to global stiffness matrix
    K(LM(1,e),LM(1,e)) = K(LM(1,e),LM(1,e)) + ke(1,1);      
    K(LM(1,e),LM(2,e)) = K(LM(1,e),LM(2,e)) + ke(1,2);      
    K(LM(2,e),LM(1,e)) = K(LM(2,e),LM(1,e)) + ke(2,1);      
    K(LM(2,e),LM(2,e)) = K(LM(2,e),LM(2,e)) + ke(2,2);      
end
K(1,1) = 1;
K(1,2:nelem+1) = 0;
K(2:nelem+1,1) = 0;

K(nelem+1, nelem+1) = 1;
K(nelem+1,1:nelem) = 0;
K(1:nelem,nelem+1) = 0;
K;
%-------------------------------------------------------------------------%
%------------------------GLOBAL LOAD VECTOR-------------------------------%
F = zeros(nelem+1,1);
for e=1:1:nelem,
    fe = elemload1D(ch,xnodes(e),xnodes(e+1));
    F(LM(1,e)) = F(LM(1,e)) + fe(1);
    F(LM(2,e)) = F(LM(2,e)) + fe(2);
end
F(1) = 0;
F(nelem+1)=0;
F;
%-------------------------------------------------------------------------%
%------------------------------FE SOLUTION--------------------------------%
Q = K\F;
%-------------------------------------------------------------------------%
if err==1,
    eta = estimator(xnodes,Q,nelem);
end

if err==2
    eta = averaging(xnodes,Q,nelem,LM);
end

etamax = max(eta);
if err==1,
    EN = norm(eta)/pi;
end
if err==2
    EN = norm(eta);
end
if EN <= TOL,
    break
end    
m = 0;

xnodesad = xnodes;
for i=1:1:nelem,
    if eta(i) >= 0.5*etamax,
        xnew = (xnodes(i)+xnodes(i+1))/2;
        xnodesad = insert(nelem+m,i,xnew,xnodesad,m);
        m = m + 1;
    end
end    

xnodes = xnodesad;
nelem = nelem + m;   
end
Q
%--------------------------PLOTTING FE SOLUTION---------------------------%
nelem
subplot(211), plot(xnodes,Q,'xr'), title('u')
hold on
%----------------------------PLOTTING EXACT SOLUTION----------------------%
syms x
uexact = (1-x)*(atan(100*(x-0.36388)) + atan(100*0.36388));    %exact U
for i=1:201;
		x=0+(i-1)*1/200;
		uexplot(i)=eval(uexact);
end
	xplot=[0:1/200:1];
	subplot(211), plot(xplot,uexplot,':')
%-------------------------------------------------------------------------%
%%%%%%%%%%%%%%%%%%%%error in ENERGY norm%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
syms x z
U_x = (1-x)*(atan(100*(x-0.36388)) + atan(100*0.36388));    %exact U
k_x = 1/100 + 100*(x-0.36388)^2;    %k(x)
DU_x = diff(U_x,x);

e_hsqEN = 0;
for ielem=1:1:N,
    k_z = subs(k_x,x,((xnodes(ielem+1)+xnodes(ielem))+z*(xnodes(ielem+1)-xnodes(ielem)))/2);
    DU_z = subs(DU_x,x,((xnodes(ielem+1)+xnodes(ielem))+z*(xnodes(ielem+1)-xnodes(ielem)))/2);
    
    U_z = subs(U_x,x,((xnodes(ielem+1)+xnodes(ielem))+z*(xnodes(ielem+1)-xnodes(ielem)))/2);    %coordinate transformation to local coordinates
    %U_zin = inline(char(U_z));
    U_z1 = Q(ielem);
    U_z2 = Q(ielem+1);
    
    Dv_h_x = (U_z2 - U_z1)/(xnodes(ielem+1)-xnodes(ielem));
    Dv_h_z = subs(Dv_h_x,x,((xnodes(ielem+1)+xnodes(ielem))+z*(xnodes(ielem+1)-xnodes(ielem)))/2);
    ENintegrand = k_z*(xnodes(ielem+1)-xnodes(ielem))/2*(DU_z - Dv_h_z)^2;
    ENint = inline(char(ENintegrand));
    
    
        ENintz = vectorize(inline(char(ENintegrand)));  %
        intz = quad(ENintz,-1,1);                       % Matlab integration
        e_hsqEN = e_hsqEN + intz;                       %
    
end

syms x z
U_x = (1-x)*(atan(100*(x-0.36388)) + atan(100*0.36388));    %exact U
k_x = 1/100 + 100*(x-0.36388)^2;    %k(x)
DU_x = diff(U_x,x);

for i=1:201;
		x=0+(i-1)*1/200;
		y(i)=eval(DU_x);
end
xplot=[0:1/200:1];
subplot(212), plot(xplot,y),title('du/dx');
hold on

    E_hEN = sqrt(e_hsqEN)
 
